@@ -19,13 +19,17 @@ def argparse():
         help="Path to the excel file (.xlsx) or table (.txt).")
 
     jobsetup_opts.add_option("--fragment",
-        action="store", dest="fraglen", type="int", default=500, metavar='fragment-length',
+        action="store", dest="fraglen", type="string", default="", metavar='fragment-length',
         help="Approximate fragment length. Default is 500. For shorter fragments, try 250.")
 
     jobsetup_opts.add_option("--overlap",
         action="store", dest="overlap", type="int", default=50, metavar='overlap-length',
         help="Sequence is extended by this amount on either side of slices. Default is 50.")
 
+    jobsetup_opts.add_option("--exhaustive",
+        action="store_true", dest="exhaustive", default=False,
+        help="Run every protein in column A against every protein in column B.")
+
     jobsetup_opts.add_option("--dimerize",
         action="store", dest="dimerize", type="string", default="", metavar='dimerize-id',
         help="Uniprot ID to dimerize. Alternatively, provide a text file (.txt) with a single column list of uniprot IDs to dimerize.")
@@ -40,7 +44,7 @@ def argparse():
 
     jobsetup_opts.add_option("--consider",
         action="store", dest="consider", type="string", default="", metavar='sequence-to-consider',
-        help="Uniprot ID and sequence range to consider. Example: \"Q86VS8/1/200\" only considers amino acids 1-200 for uniprot ID Q86VS8.")
+        help="Uniprot ID and sequence range to consider. Example: \"Q86VS8/1/200\" only considers amino acids 1-200 for uniprot ID Q86VS8. Alternatively, provide a text file with a single column list of sequences to consider with a similar syntax (id/start/end).")
 
     jobsetup_opts.add_option("--alphafold_exec",
         action="store", dest="alphafold_exec", type="string", default="colabfold2", metavar='executable',
@@ -84,6 +88,10 @@ def argparse():
         action="store", dest="threshold", type="float", default=-1, metavar="threshold",
         help="Summarize results for top hits (above the threshold specified here) in tables and pdfs.")
 
+    analyze_opts.add_option("--show_all",
+        action="store_true", dest="showall", default=False,
+        help="Summarize results for all hits in tables and pdfs.")
+
     analyze_opts.add_option("--write_table",
         action="store_true", dest="writetable", default=False,
         help="Write the full results as a table ranked by iptm.")
