@@ -11,7 +11,7 @@ def getuni(ACCESSION):
     NAME = ENTRY['uniProtkbId']
     return(NAME, SEQ)
 
-def getinteractors(file,filetype, columnA, columnB, focus):
+def getinteractors(file,filetype, columnA, columnB, focus, exhaustive):
     
     print("\n>> Reading table...\n")
     if filetype == "table":
@@ -24,7 +24,7 @@ def getinteractors(file,filetype, columnA, columnB, focus):
 
     if focus != "":
         if focus not in Ainteractors or focus not in Binteractors:
-            print("!! Warning: the uniprot ID " + focus + " was not found.\n")
+            sys.exit("!! Error: the uniprot ID " + focus + " to focus on was not found.\n")
         Ainteractors_fixed = []
         Binteractors_fixed = []
         for A, B in zip(Ainteractors,Binteractors):
@@ -34,13 +34,24 @@ def getinteractors(file,filetype, columnA, columnB, focus):
             else:
                 Ainteractors_fixed.append(A)
                 Binteractors_fixed.append(B)
-        return(Ainteractors_fixed, Binteractors_fixed)
-    else:
-        return(Ainteractors, Binteractors)
+        Ainteractors = Ainteractors_fixed
+        Binteractors = Binteractors_fixed
 
-def getfastas_writecommands(Ainteractors, Binteractors, consideruniprot,considerstart,considerend, split=True,fraglen=500,overlap=50,dimerize="",dimerize_all=False,dimerize_except="",write=True,alphafold_exec="colabfold2"):
-    
-    #print("\nStarting...\n")
+    if exhaustive:
+        Ainteractors = list(set(Ainteractors))
+        Binteractors = list(set(Binteractors))
+        Ainteractors_exh = []
+        Binteractors_exh = []
+        for A in Ainteractors:
+            for B in Binteractors:
+                Ainteractors_exh.append(A)
+                Binteractors_exh.append(B)
+        Ainteractors = Ainteractors_exh
+        Binteractors = Binteractors_exh
+
+    return(Ainteractors, Binteractors)
+
+def getfastas_writecommands(Ainteractors, Binteractors, consideruniprot,considerstart,considerend, split=True,fraguniprot=0,fraglen=500,overlap=50,dimerize="",dimerize_all=False,dimerize_except="",write=True,alphafold_exec="colabfold2"):
     
     dimerizelst = []
     dontdimerizelst = []
@@ -62,10 +73,6 @@ def getfastas_writecommands(Ainteractors, Binteractors, consideruniprot,consider
             dimerizelst = [dimerize]
             print(">> Dimerizing uniprot ID " + dimerize + "...\n")
 
-
-    if consideruniprot != "":
-        print(">> Only considering " + str(considerstart+1) + "-" + str(considerend+1) + " for uniprot ID "+consideruniprot+"...\n")
-    
     #remove duplicates
     repeating_indices = []
     for i, pair in enumerate(zip(Ainteractors, Binteractors)):
@@ -79,20 +86,19 @@ def getfastas_writecommands(Ainteractors, Binteractors, consideruniprot,consider
     Ainteractors = [j for i, j in enumerate(Ainteractors) if i not in repeating_indices]
     Binteractors = [j for i, j in enumerate(Binteractors) if i not in repeating_indices]
     
-    print(">> Removed " + str(len(repeating_indices)) + " duplicates...\n")
+    if len(Ainteractors) > 1 and len(Binteractors) > 1:
+        print(">> Removed " + str(len(repeating_indices)) + " duplicates...\n")
     
     os.makedirs('fastas', exist_ok=True)
     os.makedirs('results', exist_ok=True)
 
     for fname in os.listdir('fastas'):
         if fname.endswith('.fasta'):
-            print("!! Warning: it looks like fasta files already exist in the fastas folder.\n")
-            break
+            sys.exit(">> Error: it looks like fasta files already exist in the fastas folder.\n")
 
     colabfoldcommand = []
     unilst = []
-    Anamefail = []
-    Bnamefail = []
+    failed = []
 
     if len(dontdimerizelst) != 0:
         alluniprots = list(set(list(Ainteractors+Binteractors)))
@@ -100,17 +106,38 @@ def getfastas_writecommands(Ainteractors, Binteractors, consideruniprot,consider
 
     print(">> Getting sequences...\n")
 
+    Unis_found = []
+    Names_found = []
+    Seqs_found = []
+
+    notconsidered = consideruniprot #fully populated and will be removed one by one to check that all were found
     for A, B in zip(Ainteractors, Binteractors):
 
         try:
-            Aname,Aseq=getuni(A)
+            if A in Unis_found:
+                Aindex = Unis_found.index(A)
+                Aname = Names_found[Aindex]
+                Aseq = Seqs_found[Aindex]
+            else:
+                Aname,Aseq=getuni(A)
+                Unis_found.append(A)
+                Names_found.append(Aname)
+                Seqs_found.append(Aseq)
         except:
-            Anamefail.append(A)
+            failed.append(A)
             continue
         try:
-            Bname,Bseq=getuni(B)
+            if B in Unis_found:
+                Bindex = Unis_found.index(B)
+                Bname = Names_found[Bindex]
+                Bseq = Seqs_found[Bindex]
+            else:
+                Bname,Bseq=getuni(B)
+                Unis_found.append(B)
+                Names_found.append(Bname)
+                Seqs_found.append(Bseq)
         except:
-            Anamefail.append(B)
+            failed.append(B)
             continue
             
         unilst.append(A + ":" + Aname)
@@ -118,30 +145,59 @@ def getfastas_writecommands(Ainteractors, Binteractors, consideruniprot,consider
 
         addmeA=0
         addmeB=0
-        if consideruniprot != "":
-            if consideruniprot == A:
-                if considerend > len(Aseq):
-                    sys.exit(">> Error: last amino acid to consider is outside the protein length. Maximum is " + str(len(Aseq)+1) + "\n")
-                Aseq = Aseq[considerstart:considerend]
-                addmeA = considerstart
-            if consideruniprot == B:
-                if considerend > len(Bseq):
-                    sys.exit(">> Error: last amino acid to consider is outside the protein length. Maximum is " + str(len(Bseq)+1) + "\n")
-                Bseq = Bseq[considerstart:considerend]
-                addmeB = considerstart
+        if len(consideruniprot) > 0:
+
+            if A in consideruniprot:
+
+                a = consideruniprot.index(A)
+                if considerend[a] > len(Aseq):
+                    sys.exit(">> Error: last amino acid to consider is outside the protein length for " + consideruniprot[a] + ". Maximum is " + str(len(Aseq)+1) + ".\n")
+                Aseq = Aseq[considerstart[a]:considerend[a]]
+                addmeA = considerstart[a]
+
+                if A in notconsidered:
+                    notconsidered.remove(A) 
+
+            if B in consideruniprot:
+
+                b = consideruniprot.index(B)
+                if considerend[b] > len(Bseq):
+                    sys.exit(">> Error: last amino acid to consider is outside the protein length for " + consideruniprot[b] + ". Maximum is " + str(len(Bseq)+1) + ".\n")
+                Bseq = Bseq[considerstart[b]:considerend[b]]
+                addmeB = considerstart[b]
+
+                if B in notconsidered:
+                    notconsidered.remove(B) 
         
         if split:
-            #print(Aname)
-            Anamelst, Aseqlst = splitfasta(Aname,Aseq,fraglen,overlap,addmeA)
-            #print(Bname)
-            Bnamelst, Bseqlst = splitfasta(Bname,Bseq,fraglen,overlap,addmeB)
+
+            if type(fraglen) == list:
+                if A in fraguniprot:
+                    fraglen_cur=fraglen[fraguniprot.index(A)]
+                else:
+                    fraglen_cur = 500
+            else:
+                fraglen_cur = fraglen
+
+            Anamelst, Aseqlst = splitfasta(Aname,Aseq,fraglen_cur,overlap,addmeA)
+
+            if type(fraglen) == list:
+                if B in fraguniprot:
+                    fraglen_cur=fraglen[fraguniprot.index(B)]
+                else:
+                    fraglen_cur = 500
+            else:
+                fraglen_cur = fraglen
+
+            Bnamelst, Bseqlst = splitfasta(Bname,Bseq,fraglen_cur,overlap,addmeB)
+
         else:
+
             Anamelst = [Aname]
             Aseqlst = [Aseq]
             Bnamelst = [Bname]
             Bseqlst = [Bseq]
             
-        #print(Anamelst, Aseqlst)
         
         for An, As in zip(Anamelst, Aseqlst):
             
@@ -165,13 +221,17 @@ def getfastas_writecommands(Ainteractors, Binteractors, consideruniprot,consider
                             f.write(">"+Bn+"\n"+Bs+"\n")
 
                 colabfoldcommand.append(alphafold_exec + " " + fastaname_minimal + " --output=results/" + An + "-" + Bn)
-    
-    failed = Anamefail + Bnamefail
+
     if len(failed) > 0:
-        print(">> Warning: there was a problem getting the Uniprot data for accessions:\n")
+        failed = list(set(failed))
+        print("!! Warning: there was a problem getting the Uniprot data for accessions:\n")
         for f in failed:
             print("·· \""+ str(f) + "\"\n")
 
+    if len(notconsidered) > 0:
+        for c in notconsidered:
+            print("!! Warning: you asked to consider a region within " + c + ", but it wasn't in your input.\n")
+
     colabfoldcommand=list(dict.fromkeys(colabfoldcommand)) 
 
     if write:
@@ -253,7 +313,7 @@ def findunfinished(alphafold_exec, write=True):
         resultdir = "results" + str(fastapath).split("fastas")[1][:-6]
 
         if not os.path.exists(resultdir) and not warned:
-            print("\n>> Warning: could not find one of the directories. Make sure this the right directory where the Alphafold jobs were run.")
+            print("\n!! Warning: could not find one of the directories. Make sure this the right directory where the Alphafold jobs were run.")
             warned=True
         
         found = False
